extern crate chrono;
extern crate colored;
extern crate dirs;
extern crate fern;
extern crate getopts;
#[macro_use]
extern crate log;
extern crate mio;
extern crate mio_child_process;
extern crate serde;
#[macro_use]
extern crate serde_derive;
extern crate signal_hook;
extern crate toml;

mod app;
mod apps;
mod config;
mod logger;
mod proc;
mod result;
mod run;

use std::path::PathBuf;

use apps::Apps;
use config::Config;
use proc::Proc;
use result::Result;
use run::run;

fn main() {
	let config = Config::from_args();

	if let Err(e) = logger::setup(config.log_file, config.log_level) {
		println!("FATAL: Error initializing logger: {:?}", e);
		std::process::exit(1);
	}

	let mut apps_file = config.config_directory.clone();
	apps_file.push("apps.toml");

	match startup(apps_file) {
		Ok(p) => {
			info!("shutting down");
			for mut proc in p {
				proc.stop();
			}
		}
		Err(e) => {
			error!("FATAL: {:?}", e);
			std::process::exit(1);
		}
	};
}

fn startup(apps_file: PathBuf) -> Result<Vec<Proc>> {
	let apps = Apps::load(apps_file)?;
	let (procs, holds, poll) = apps.start();
	let procs = run(procs, holds, poll)?;
	Ok(procs)
}
