use std::convert::TryInto;
use std::process::{Command, Stdio};

use mio::Token;
use mio_child_process::{CommandAsync, Process};

use crate::app::App;
use crate::result::Result;

pub struct Proc {
	pub app: App,
	pub process: Process,
	pub token: Token,
}

impl Proc {
	pub fn start(app: App) -> Result<Self> {
		info!("starting application {}", &app.name);

		let mut command = Command::new(&app.command);
		command.stdout(Stdio::piped());
		command.stderr(Stdio::piped());

		let args = app.args.as_ref();
		command.args(args.unwrap_or(&Vec::new()));

		let process = command.spawn_async()?;
		let token = Token(process.id().try_into()?);

		Ok(Proc {
			app,
			process,
			token,
		})
	}

	pub fn stop(&mut self) {
		info!("stopping {}", &self.app.name);
		let _ = self.process.kill();
	}
}
